// Generated by CoffeeScript 2.7.0
(function() {
  (function(window, factory) {
    if (typeof define === 'function' && define.amd) {
      return define(['jquery'], function(jQuery) {
        return window.Tour = factory(jQuery);
      });
    } else if (typeof exports === 'object') {
      return module.exports = factory(require('jquery'));
    } else {
      return window.Tour = factory(window.jQuery);
    }
  })(window, function($) {
    var Tour, document;
    document = window.document;
    Tour = class Tour {
      constructor(options) {
        var storage;
        this._showPopoverAndOverlay = this._showPopoverAndOverlay.bind(this);
        try {
          storage = window.localStorage;
        } catch (error) {
          // localStorage may be unavailable due to security settings
          storage = false;
        }
        this._options = $.extend({
          name: 'tour',
          steps: [],
          container: 'body',
          autoscroll: true,
          keyboard: true,
          storage: storage,
          debug: false,
          backdrop: false,
          backdropContainer: 'body',
          backdropPadding: 0,
          redirect: true,
          orphan: false,
          duration: false,
          delay: false,
          basePath: '',
          template: '<div class="popover" role="tooltip"> <div class="arrow"></div> <h3 class="popover-title"></h3> <div class="popover-content"></div> <div class="popover-navigation"> <div class="btn-group"> <button class="btn btn-sm btn-default" data-role="prev">&laquo; Prev</button> <button class="btn btn-sm btn-default" data-role="next">Next &raquo;</button> <button class="btn btn-sm btn-default" data-role="pause-resume" data-pause-text="Pause" data-resume-text="Resume">Pause</button> </div> <button class="btn btn-sm btn-default" data-role="end">End tour</button> </div> </div>',
          afterSetState: function(key, value) {},
          afterGetState: function(key, value) {},
          afterRemoveState: function(key) {},
          onStart: function(tour) {},
          onEnd: function(tour) {},
          onShow: function(tour) {},
          onShown: function(tour) {},
          onHide: function(tour) {},
          onHidden: function(tour) {},
          onNext: function(tour) {},
          onPrev: function(tour) {},
          onPause: function(tour, duration) {},
          onResume: function(tour, duration) {},
          onRedirectError: function(tour) {}
        }, options);
        this._force = false;
        this._inited = false;
        this._current = null;
        this.backdrops = [];
        this;
      }

      // Add multiple steps
      addSteps(steps) {
        var j, len, step;
        for (j = 0, len = steps.length; j < len; j++) {
          step = steps[j];
          this.addStep(step);
        }
        return this;
      }

      // Add a new step
      addStep(step) {
        this._options.steps.push(step);
        return this;
      }

      // Get a step by its indice
      getStep(i) {
        if (this._options.steps[i] != null) {
          return $.extend({
            id: `step-${i}`,
            path: '',
            host: '',
            placement: 'right',
            title: '',
            content: '<p></p>', // no empty as default, otherwise popover won't show up
            next: i === this._options.steps.length - 1 ? -1 : i + 1,
            prev: i - 1,
            animation: true,
            container: this._options.container,
            autoscroll: this._options.autoscroll,
            backdrop: this._options.backdrop,
            backdropContainer: this._options.backdropContainer,
            backdropPadding: this._options.backdropPadding,
            redirect: this._options.redirect,
            reflexElement: this._options.steps[i].element,
            backdropElement: this._options.steps[i].element,
            orphan: this._options.orphan,
            duration: this._options.duration,
            delay: this._options.delay,
            template: this._options.template,
            onShow: this._options.onShow,
            onShown: this._options.onShown,
            onHide: this._options.onHide,
            onHidden: this._options.onHidden,
            onNext: this._options.onNext,
            onPrev: this._options.onPrev,
            onPause: this._options.onPause,
            onResume: this._options.onResume,
            onRedirectError: this._options.onRedirectError
          }, this._options.steps[i]);
        }
      }

      // Setup event bindings and continue a tour that has already started
      init(force) {
        this._force = force;
        if (this.ended()) {
          this._debug('Tour ended, init prevented.');
          return this;
        }
        this.setCurrentStep();
        this._initMouseNavigation();
        this._initKeyboardNavigation();
        // Reshow popover on window resize using debounced resize
        this._onResize(() => {
          return this.showStep(this._current);
        });
        this._onScroll(() => {
          return this._showPopoverAndOverlay(this._current);
        });
        if (this._current !== null) {
          // Continue a tour that had started on a previous page load
          this.showStep(this._current);
        }
        this._inited = true;
        return this;
      }

      // Start tour from current step
      start(force = false) {
        var promise;
        if (!this._inited) { // Backward compatibility
          this.init(force);
        }
        if (this._current === null) {
          promise = this._makePromise(this._options.onStart != null ? this._options.onStart(this) : void 0);
          this._callOnPromiseDone(promise, this.showStep, 0);
        }
        return this;
      }

      // Hide current step and show next step
      next() {
        var promise;
        promise = this.hideStep(this._current, this._current + 1);
        return this._callOnPromiseDone(promise, this._showNextStep);
      }

      // Hide current step and show prev step
      prev() {
        var promise;
        promise = this.hideStep(this._current, this._current - 1);
        return this._callOnPromiseDone(promise, this._showPrevStep);
      }

      goTo(i) {
        var promise;
        promise = this.hideStep(this._current, i);
        return this._callOnPromiseDone(promise, this.showStep, i);
      }

      // End tour
      end() {
        var endHelper, promise;
        endHelper = (e) => {
          $(document).off(`click.tour-${this._options.name}`);
          $(document).off(`keyup.tour-${this._options.name}`);
          $(window).off(`resize.tour-${this._options.name}`);
          $(window).off(`scroll.tour-${this._options.name}`);
          this._setState('end', 'yes');
          this._inited = false;
          this._force = false;
          this._clearTimer();
          if (this._options.onEnd != null) {
            return this._options.onEnd(this);
          }
        };
        promise = this.hideStep(this._current);
        return this._callOnPromiseDone(promise, endHelper);
      }

      // Verify if tour is enabled
      ended() {
        return !this._force && !!this._getState('end');
      }

      // Restart tour
      restart() {
        this._removeState('current_step');
        this._removeState('end');
        this._removeState('redirect_to');
        return this.start();
      }

      // Pause step timer
      pause() {
        var step;
        step = this.getStep(this._current);
        if (!(step && step.duration)) {
          return this;
        }
        this._paused = true;
        this._duration -= new Date().getTime() - this._start;
        window.clearTimeout(this._timer);
        this._debug(`Paused/Stopped step ${this._current + 1} timer (${this._duration} remaining).`);
        if (step.onPause != null) {
          return step.onPause(this, this._duration);
        }
      }

      // Resume step timer
      resume() {
        var step;
        step = this.getStep(this._current);
        if (!(step && step.duration)) {
          return this;
        }
        this._paused = false;
        this._start = new Date().getTime();
        this._duration = this._duration || step.duration;
        this._timer = window.setTimeout(() => {
          if (this._isLast()) {
            return this.next();
          } else {
            return this.end();
          }
        }, this._duration);
        this._debug(`Started step ${this._current + 1} timer with duration ${this._duration}`);
        if ((step.onResume != null) && this._duration !== step.duration) {
          return step.onResume(this, this._duration);
        }
      }

      // Hide the specified step
      hideStep(i, iNext) {
        var hideDelay, hideStepHelper, promise, step;
        step = this.getStep(i);
        if (!step) {
          return;
        }
        this._clearTimer();
        // If onHide returns a promise, let's wait until it's done to execute
        promise = this._makePromise(step.onHide != null ? step.onHide(this, i) : void 0);
        hideStepHelper = (e) => {
          var $element, next_step;
          $element = $(step.element);
          if (!($element.data('bs.popover') || $element.data('popover'))) {
            $element = $('body');
          }
          $element.popover('destroy').removeClass(`tour-${this._options.name}-element tour-${this._options.name}-${i}-element`).removeData('bs.popover');
          if (step.reflex) {
            $(step.reflexElement).removeClass('tour-step-element-reflex').off(`${this._reflexEvent(step.reflex)}.tour-${this._options.name}`);
          }
          if (step.backdrop) {
            next_step = (iNext != null) && this.getStep(iNext);
            if (!next_step || !next_step.backdrop || next_step.backdropElement !== step.backdropElement) {
              this._hideOverlayElement(step);
            }
          }
          if (step.onHidden != null) {
            return step.onHidden(this);
          }
        };
        hideDelay = step.delay.hide || step.delay;
        if ({}.toString.call(hideDelay) === '[object Number]' && hideDelay > 0) {
          this._debug(`Wait ${hideDelay} milliseconds to hide the step ${this._current + 1}`);
          window.setTimeout(() => {
            return this._callOnPromiseDone(promise, hideStepHelper);
          }, hideDelay);
        } else {
          this._callOnPromiseDone(promise, hideStepHelper);
        }
        return promise;
      }

      // Show the specified step
      showStep(i) {
        var path, promise, showDelay, showStepHelper, skipToPrevious, step;
        if (this.ended()) {
          this._debug('Tour ended, showStep prevented.');
          return this;
        }
        step = this.getStep(i);
        if (!step) {
          return;
        }
        skipToPrevious = i < this._current;
        // If onShow returns a promise, let's wait until it's done to execute
        promise = this._makePromise(step.onShow != null ? step.onShow(this, i) : void 0);
        this.setCurrentStep(i);
        // Support string or function for path
        path = (function() {
          switch ({}.toString.call(step.path)) {
            case '[object Function]':
              return step.path();
            case '[object String]':
              return this._options.basePath + step.path;
            default:
              return step.path;
          }
        }).call(this);
        // Redirect to step path if not already there
        if (step.redirect && this._isRedirect(step.host, path, document.location)) {
          this._redirect(step, i, path);
          if (!this._isJustPathHashDifferent(step.host, path, document.location)) {
            return;
          }
        }
        showStepHelper = (e) => {
          // Skip if step is orphan and orphan options is false
          if (this._isOrphan(step)) {
            if (step.orphan === false) {
              this._debug(`Skip the orphan step ${this._current + 1}.
Orphan option is false and the element does not exist or is hidden.`);
              if (skipToPrevious) {
                this._showPrevStep();
              } else {
                this._showNextStep();
              }
              return;
            }
            this._debug(`Show the orphan step ${this._current + 1}. Orphans option is true.`);
          }
          // Show backdrop
          // @_showBackdrop(step) if step.backdrop
          if (step.autoscroll) {
            this._scrollIntoView(i);
          } else {
            this._showPopoverAndOverlay(i);
          }
          if (step.duration) {
            // Play step timer
            return this.resume();
          }
        };
        showDelay = step.delay.show || step.delay;
        if ({}.toString.call(showDelay) === '[object Number]' && showDelay > 0) {
          this._debug(`Wait ${showDelay} milliseconds to show the step ${this._current + 1}`);
          window.setTimeout(() => {
            return this._callOnPromiseDone(promise, showStepHelper);
          }, showDelay);
        } else {
          this._callOnPromiseDone(promise, showStepHelper);
        }
        return promise;
      }

      getCurrentStep() {
        return this._current;
      }

      // Setup current step variable
      setCurrentStep(value) {
        if (value != null) {
          this._current = value;
          this._setState('current_step', value);
        } else {
          this._current = this._getState('current_step');
          this._current = this._current === null ? null : parseInt(this._current, 10);
        }
        return this;
      }

      // Manually trigger a redraw on the overlay element
      redraw() {
        return this._showOverlayElement(this.getStep(this.getCurrentStep()));
      }

      // Set a state in storage
      _setState(key, value) {
        var e, keyName;
        if (this._options.storage) {
          keyName = `${this._options.name}_${key}`;
          try {
            this._options.storage.setItem(keyName, value);
          } catch (error) {
            e = error;
            if (e.code === DOMException.QUOTA_EXCEEDED_ERR) {
              this._debug('LocalStorage quota exceeded. State storage failed.');
            }
          }
          return this._options.afterSetState(keyName, value);
        } else {
          if (this._state == null) {
            this._state = {};
          }
          return this._state[key] = value;
        }
      }

      // Remove the current state from the storage layer
      _removeState(key) {
        var keyName;
        if (this._options.storage) {
          keyName = `${this._options.name}_${key}`;
          this._options.storage.removeItem(keyName);
          return this._options.afterRemoveState(keyName);
        } else {
          if (this._state != null) {
            return delete this._state[key];
          }
        }
      }

      // Get the current state from the storage layer
      _getState(key) {
        var keyName, value;
        if (this._options.storage) {
          keyName = `${this._options.name}_${key}`;
          value = this._options.storage.getItem(keyName);
        } else {
          if (this._state != null) {
            value = this._state[key];
          }
        }
        if (value === void 0 || value === 'null') {
          value = null;
        }
        this._options.afterGetState(key, value);
        return value;
      }

      // Show next step
      _showNextStep() {
        var promise, showNextStepHelper, step;
        step = this.getStep(this._current);
        showNextStepHelper = (e) => {
          return this.showStep(step.next);
        };
        promise = this._makePromise(step.onNext != null ? step.onNext(this) : void 0);
        return this._callOnPromiseDone(promise, showNextStepHelper);
      }

      // Show prev step
      _showPrevStep() {
        var promise, showPrevStepHelper, step;
        step = this.getStep(this._current);
        showPrevStepHelper = (e) => {
          return this.showStep(step.prev);
        };
        promise = this._makePromise(step.onPrev != null ? step.onPrev(this) : void 0);
        return this._callOnPromiseDone(promise, showPrevStepHelper);
      }

      // Print message in console
      _debug(text) {
        if (this._options.debug) {
          return window.console.log(`Bootstrap Tour '${this._options.name}' | ${text}`);
        }
      }

      // Check if step path equals current document path
      _isRedirect(host, path, location) {
        var currentPath;
        if ((host != null) && host !== '' && (({}.toString.call(host) === '[object RegExp]' && !host.test(location.origin)) || ({}.toString.call(host) === '[object String]' && this._isHostDifferent(host, location)))) {
          return true;
        }
        currentPath = [location.pathname, location.search, location.hash].join('');
        return (path != null) && path !== '' && (({}.toString.call(path) === '[object RegExp]' && !path.test(currentPath)) || ({}.toString.call(path) === '[object String]' && this._isPathDifferent(path, currentPath)));
      }

      _isHostDifferent(host, location) {
        switch ({}.toString.call(host)) {
          case '[object RegExp]':
            return !host.test(location.origin);
          case '[object String]':
            return this._getProtocol(host) !== this._getProtocol(location.href) || this._getHost(host) !== this._getHost(location.href);
          default:
            return true;
        }
      }

      _isPathDifferent(path, currentPath) {
        return this._getPath(path) !== this._getPath(currentPath) || !this._equal(this._getQuery(path), this._getQuery(currentPath)) || !this._equal(this._getHash(path), this._getHash(currentPath));
      }

      _isJustPathHashDifferent(host, path, location) {
        var currentPath;
        if ((host != null) && host !== '') {
          if (this._isHostDifferent(host, location)) {
            return false;
          }
        }
        currentPath = [location.pathname, location.search, location.hash].join('');
        if ({}.toString.call(path) === '[object String]') {
          return this._getPath(path) === this._getPath(currentPath) && this._equal(this._getQuery(path), this._getQuery(currentPath)) && !this._equal(this._getHash(path), this._getHash(currentPath));
        }
        return false;
      }

      // Execute the redirect
      _redirect(step, i, path) {
        var href;
        if ($.isFunction(step.redirect)) {
          return step.redirect.call(this, path);
        } else {
          href = {}.toString.call(step.host) === '[object String]' ? `${step.host}${path}` : path;
          this._debug(`Redirect to ${href}`);
          if (this._getState('redirect_to') === `${i}`) {
            this._debug(`Error redirection loop to ${path}`);
            this._removeState('redirect_to');
            if (step.onRedirectError != null) {
              return step.onRedirectError(this);
            }
          } else {
            this._setState('redirect_to', `${i}`);
            return document.location.href = href;
          }
        }
      }

      _isOrphan(step) {
        // Do not check for is(':hidden') on svg elements. jQuery does not work properly on svg.
        return (step.element == null) || !$(step.element).length || $(step.element).is(':hidden') && ($(step.element)[0].namespaceURI !== 'http://www.w3.org/2000/svg');
      }

      _isLast() {
        return this._current < this._options.steps.length - 1;
      }

      _showPopoverAndOverlay(i) {
        var step;
        if (this.getCurrentStep() !== i || this.ended()) {
          return;
        }
        step = this.getStep(i);
        if (step.backdrop) {
          this._showOverlayElement(step);
        }
        this._showPopover(step, i);
        if (step.onShown != null) {
          step.onShown(this);
        }
        return this._debug(`Step ${this._current + 1} of ${this._options.steps.length}`);
      }

      // Show step popover
      _showPopover(step, i) {
        var $element, $tip, isOrphan, options, shouldAddSmart;
        // Remove previously existing tour popovers. This prevents displaying of
        // multiple inactive popovers when user navigates the tour too quickly.
        $(`.tour-${this._options.name}`).remove();
        options = $.extend({}, this._options);
        isOrphan = this._isOrphan(step);
        step.template = this._template(step, i);
        if (isOrphan) {
          step.element = 'body';
          step.placement = 'top';
        }
        $element = $(step.element);
        $element.addClass(`tour-${this._options.name}-element tour-${this._options.name}-${i}-element`);
        if (step.options) {
          $.extend(options, step.options);
        }
        if (step.reflex && !isOrphan) {
          $(step.reflexElement).addClass('tour-step-element-reflex').off(`${this._reflexEvent(step.reflex)}.tour-${this._options.name}`).on(`${this._reflexEvent(step.reflex)}.tour-${this._options.name}`, () => {
            if (this._isLast()) {
              return this.next();
            } else {
              return this.end();
            }
          });
        }
        shouldAddSmart = step.smartPlacement === true && step.placement.search(/auto/i) === -1;
        $element.popover({
          placement: shouldAddSmart ? `auto ${step.placement}` : step.placement,
          trigger: 'manual',
          title: step.title,
          content: step.content,
          html: true,
          sanitize: false,
          animation: step.animation,
          container: step.container,
          template: step.template,
          selector: step.element
        }).popover('show');
        // Tip adjustment
        $tip = $element.data('bs.popover') ? $element.data('bs.popover').tip() : $element.data('popover').tip();
        $tip.attr('id', step.id);
        if ($element.css('position') === 'fixed') {
          $tip.css('position', 'fixed');
        }
        this._reposition($tip, step);
        if (isOrphan) {
          return this._center($tip);
        }
      }

      // Get popover template
      _template(step, i) {
        var $navigation, $next, $prev, $resume, $template, template;
        template = step.template;
        if (this._isOrphan(step) && {}.toString.call(step.orphan) !== '[object Boolean]') {
          template = step.orphan;
        }
        $template = $.isFunction(template) ? $(template(i, step)) : $(template);
        $navigation = $template.find('.popover-navigation');
        $prev = $navigation.find('[data-role="prev"]');
        $next = $navigation.find('[data-role="next"]');
        $resume = $navigation.find('[data-role="pause-resume"]');
        if (this._isOrphan(step)) {
          $template.addClass('orphan');
        }
        $template.addClass(`tour-${this._options.name} tour-${this._options.name}-${i}`);
        if (step.reflex) {
          $template.addClass(`tour-${this._options.name}-reflex`);
        }
        if (step.prev < 0) {
          $prev.addClass('disabled').prop('disabled', true).prop('tabindex', -1);
        }
        if (step.next < 0) {
          $next.addClass('disabled').prop('disabled', true).prop('tabindex', -1);
        }
        if (!step.duration) {
          $resume.remove();
        }
        return $template.clone().wrap('<div>').parent().html();
      }

      _reflexEvent(reflex) {
        if ({}.toString.call(reflex) === '[object Boolean]') {
          return 'click';
        } else {
          return reflex;
        }
      }

      // Prevent popover from crossing over the edge of the window
      _reposition($tip, step) {
        var offsetBottom, offsetHeight, offsetRight, offsetWidth, originalLeft, originalTop, tipOffset;
        offsetWidth = $tip[0].offsetWidth;
        offsetHeight = $tip[0].offsetHeight;
        tipOffset = $tip.offset();
        originalLeft = tipOffset.left;
        originalTop = tipOffset.top;
        offsetBottom = $(document).outerHeight() - tipOffset.top - $tip.outerHeight();
        if (offsetBottom < 0) {
          tipOffset.top = tipOffset.top + offsetBottom;
        }
        offsetRight = $('html').outerWidth() - tipOffset.left - $tip.outerWidth();
        if (offsetRight < 0) {
          tipOffset.left = tipOffset.left + offsetRight;
        }
        if (tipOffset.top < 0) {
          tipOffset.top = 0;
        }
        if (tipOffset.left < 0) {
          tipOffset.left = 0;
        }
        $tip.offset(tipOffset);
        // Reposition the arrow
        if (step.placement === 'bottom' || step.placement === 'top') {
          if (originalLeft !== tipOffset.left) {
            return this._replaceArrow($tip, (tipOffset.left - originalLeft) * 2, offsetWidth, 'left');
          }
        } else {
          if (originalTop !== tipOffset.top) {
            return this._replaceArrow($tip, (tipOffset.top - originalTop) * 2, offsetHeight, 'top');
          }
        }
      }

      // Center popover in the page
      _center($tip) {
        return $tip.css('top', $(window).outerHeight() / 2 - $tip.outerHeight() / 2);
      }

      // Copy pasted from bootstrap-tooltip.js with some alterations
      _replaceArrow($tip, delta, dimension, position) {
        return $tip.find('.arrow').css(position, delta ? 50 * (1 - delta / dimension) + '%' : '');
      }

      // Scroll to the popup if it is not in the viewport
      _scrollIntoView(i) {
        var $element, $window, counter, height, offsetTop, scrollTop, step, windowHeight;
        step = this.getStep(i);
        $element = $(step.element);
        if (!$element.length) {
          return this._showPopoverAndOverlay(i);
        }
        $window = $(window);
        offsetTop = $element.offset().top;
        height = $element.outerHeight();
        windowHeight = $window.height();
        scrollTop = 0;
        switch (step.placement) {
          case 'top':
            scrollTop = Math.max(0, offsetTop - (windowHeight / 2));
            break;
          case 'left':
          case 'right':
            scrollTop = Math.max(0, (offsetTop + height / 2) - (windowHeight / 2));
            break;
          case 'bottom':
            scrollTop = Math.max(0, (offsetTop + height) - (windowHeight / 2));
        }
        this._debug(`Scroll into view. ScrollTop: ${scrollTop}. Element offset: ${offsetTop}. Window height: ${windowHeight}.`);
        counter = 0;
        return $('body, html').stop(true, true).animate({
          scrollTop: Math.ceil(scrollTop)
        }, () => {
          if (++counter === 2) {
            this._showPopoverAndOverlay(i);
            return this._debug(`Scroll into view.
Animation end element offset: ${$element.offset().top}.
Window height: ${$window.height()}.`);
          }
        });
      }

      // Debounced window resize
      _onResize(callback, timeout) {
        return $(window).on(`resize.tour-${this._options.name}`, function() {
          clearTimeout(timeout);
          return timeout = setTimeout(callback, 100);
        });
      }

      // Debounced window scroll
      _onScroll(callback, timeout) {
        return $(window).on(`scroll.tour-${this._options.name}`, function() {
          clearTimeout(timeout);
          return timeout = setTimeout(callback, 100);
        });
      }

      // Event bindings for mouse navigation
      _initMouseNavigation() {
        var _this;
        _this = this;
        // Go to next step after click on element with attribute 'data-role=next'
        // Go to previous step after click on element with attribute 'data-role=prev'
        // End tour after click on element with attribute 'data-role=end'
        // Pause/resume tour after click on element with attribute 'data-role=pause-resume'
        return $(document).off(`click.tour-${this._options.name}`, `.popover.tour-${this._options.name} *[data-role='prev']`).off(`click.tour-${this._options.name}`, `.popover.tour-${this._options.name} *[data-role='next']`).off(`click.tour-${this._options.name}`, `.popover.tour-${this._options.name} *[data-role='end']`).off(`click.tour-${this._options.name}`, `.popover.tour-${this._options.name} *[data-role='pause-resume']`).on(`click.tour-${this._options.name}`, `.popover.tour-${this._options.name} *[data-role='next']`, (e) => {
          e.preventDefault();
          return this.next();
        }).on(`click.tour-${this._options.name}`, `.popover.tour-${this._options.name} *[data-role='prev']`, (e) => {
          e.preventDefault();
          if (this._current > 0) {
            return this.prev();
          }
        }).on(`click.tour-${this._options.name}`, `.popover.tour-${this._options.name} *[data-role='end']`, (e) => {
          e.preventDefault();
          return this.end();
        }).on(`click.tour-${this._options.name}`, `.popover.tour-${this._options.name} *[data-role='pause-resume']`, function(e) {
          var $this;
          e.preventDefault();
          $this = $(this);
          $this.text(_this._paused ? $this.data('pause-text') : $this.data('resume-text'));
          if (_this._paused) {
            return _this.resume();
          } else {
            return _this.pause();
          }
        });
      }

      // Keyboard navigation
      _initKeyboardNavigation() {
        if (!this._options.keyboard) {
          return;
        }
        return $(document).on(`keyup.tour-${this._options.name}`, (e) => {
          if (!e.which) {
            return;
          }
          switch (e.which) {
            case 39:
              e.preventDefault();
              if (this._isLast()) {
                return this.next();
              } else {
                return this.end();
              }
              break;
            case 37:
              e.preventDefault();
              if (this._current > 0) {
                return this.prev();
              }
          }
        });
      }

      // Checks if the result of a callback is a promise
      _makePromise(result) {
        if (result && $.isFunction(result.then)) {
          return result;
        } else {
          return null;
        }
      }

      _callOnPromiseDone(promise, cb, arg) {
        if (promise) {
          return promise.then((e) => {
            return cb.call(this, arg);
          });
        } else {
          return cb.call(this, arg);
        }
      }

      _showBackground(step, data) {
        var $backdrop, base, height, j, len, pos, ref, results, width;
        height = $(document).height();
        width = $(document).width();
        ref = ['top', 'bottom', 'left', 'right'];
        results = [];
        for (j = 0, len = ref.length; j < len; j++) {
          pos = ref[j];
          $backdrop = (base = this.backdrops)[pos] != null ? base[pos] : base[pos] = $('<div>', {
            class: `tour-backdrop ${pos}`
          });
          $(step.backdropContainer).append($backdrop);
          switch (pos) {
            case 'top':
              results.push($backdrop.height(data.offset.top > 0 ? data.offset.top : 0).width(width).offset({
                top: 0,
                left: 0
              }));
              break;
            case 'bottom':
              results.push($backdrop.offset({
                top: data.offset.top + data.height,
                left: 0
              }).height(height - (data.offset.top + data.height)).width(width));
              break;
            case 'left':
              results.push($backdrop.offset({
                top: data.offset.top,
                left: 0
              }).height(data.height).width(data.offset.left > 0 ? data.offset.left : 0));
              break;
            case 'right':
              results.push($backdrop.offset({
                top: data.offset.top,
                left: data.offset.left + data.width
              }).height(data.height).width(width - (data.offset.left + data.width)));
              break;
            default:
              results.push(void 0);
          }
        }
        return results;
      }

      _showOverlayElement(step) {
        var $backdropElement, elementData;
        $backdropElement = $(step.backdropElement);
        if ($backdropElement.length === 0) {
          elementData = {
            width: 0,
            height: 0,
            offset: {
              top: 0,
              left: 0
            }
          };
        } else {
          elementData = {
            width: $backdropElement.innerWidth(),
            height: $backdropElement.innerHeight(),
            offset: $backdropElement.offset()
          };
          $backdropElement.addClass('tour-step-backdrop');
          if (step.backdropPadding) {
            elementData = this._applyBackdropPadding(step.backdropPadding, elementData);
          }
        }
        return this._showBackground(step, elementData);
      }

      _hideOverlayElement(step) {
        var $backdrop, pos, ref;
        $(step.backdropElement).removeClass('tour-step-backdrop');
        ref = this.backdrops;
        for (pos in ref) {
          $backdrop = ref[pos];
          if ($backdrop && $backdrop.remove !== void 0) {
            $backdrop.remove();
          }
        }
        return this.backdrops = [];
      }

      _applyBackdropPadding(padding, data) {
        if (typeof padding === 'object') {
          if (padding.top == null) {
            padding.top = 0;
          }
          if (padding.right == null) {
            padding.right = 0;
          }
          if (padding.bottom == null) {
            padding.bottom = 0;
          }
          if (padding.left == null) {
            padding.left = 0;
          }
          data.offset.top = data.offset.top - padding.top;
          data.offset.left = data.offset.left - padding.left;
          data.width = data.width + padding.left + padding.right;
          data.height = data.height + padding.top + padding.bottom;
        } else {
          data.offset.top = data.offset.top - padding;
          data.offset.left = data.offset.left - padding;
          data.width = data.width + (padding * 2);
          data.height = data.height + (padding * 2);
        }
        return data;
      }

      _clearTimer() {
        window.clearTimeout(this._timer);
        this._timer = null;
        return this._duration = null;
      }

      _getProtocol(url) {
        url = url.split('://');
        if (url.length > 1) {
          return url[0];
        } else {
          return 'http';
        }
      }

      _getHost(url) {
        url = url.split('//');
        url = url.length > 1 ? url[1] : url[0];
        return url.split('/')[0];
      }

      _getPath(path) {
        return path.replace(/\/?$/, '').split('?')[0].split('#')[0];
      }

      _getQuery(path) {
        return this._getParams(path, '?');
      }

      _getHash(path) {
        return this._getParams(path, '#');
      }

      _getParams(path, start) {
        var j, len, param, params, paramsObject;
        params = path.split(start);
        if (params.length === 1) {
          return {};
        }
        params = params[1].split('&');
        paramsObject = {};
        for (j = 0, len = params.length; j < len; j++) {
          param = params[j];
          param = param.split('=');
          paramsObject[param[0]] = param[1] || '';
        }
        return paramsObject;
      }

      _equal(obj1, obj2) {
        var j, k, len, obj1Keys, obj2Keys, v;
        if ({}.toString.call(obj1) === '[object Object]' && {}.toString.call(obj2) === '[object Object]') {
          obj1Keys = Object.keys(obj1);
          obj2Keys = Object.keys(obj2);
          if (obj1Keys.length !== obj2Keys.length) {
            return false;
          }
          for (k in obj1) {
            v = obj1[k];
            if (!this._equal(obj2[k], v)) {
              return false;
            }
          }
          return true;
        } else if ({}.toString.call(obj1) === '[object Array]' && {}.toString.call(obj2) === '[object Array]') {
          if (obj1.length !== obj2.length) {
            return false;
          }
          for (k = j = 0, len = obj1.length; j < len; k = ++j) {
            v = obj1[k];
            if (!this._equal(v, obj2[k])) {
              return false;
            }
          }
          return true;
        } else {
          return obj1 === obj2;
        }
      }

    };
    return Tour;
  });

}).call(this);
